@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -23,6 +23,7 @@
 #include "dump.h"
 #include "info.h"
 #include "json.h"
+#include "util.h"
 
 #include <math.h>
 #include <stdlib.h>
@@ -43,7 +44,7 @@ static void report_start(const flvmeta_opts * opts, check_context * ctxt) {
     time_t now;
     struct tm * t;
     char datestr[128];
-    
+
     if (opts->quiet)
         return;
 
@@ -81,7 +82,7 @@ static void report_start(const flvmeta_opts * opts, check_context * ctxt) {
 }
 
 /* end the report */
-static void report_end(const flvmeta_opts * opts, check_context * ctxt, int errors, int warnings) {
+static void report_end(const flvmeta_opts * opts, check_context * ctxt, uint32 errors, uint32 warnings) {
     if (opts->quiet)
         return;
 
@@ -103,7 +104,7 @@ static void report_end(const flvmeta_opts * opts, check_context * ctxt, int erro
         printf("\n");
     }
     else {
-        printf("%d error(s), %d warning(s)\n", errors, warnings);
+        printf("%u error(s), %u warning(s)\n", errors, warnings);
     }
 }
 
@@ -133,7 +134,7 @@ static void report_print_message(
         if (opts->check_report_format == FLVMETA_FORMAT_XML) {
             /* XML report entry */
             printf("    <message level=\"%s\" code=\"%s\"", levelstr, code);
-            printf(" offset=\"%" FILE_OFFSET_PRINTF_FORMAT  "i\">", offset);
+            printf(" offset=\"%" FILE_OFFSET_PRINTF_FORMAT  "u\">", FILE_OFFSET_PRINTF_TYPE(offset));
             printf("%s</message>\n", message);
         }
         else if (opts->check_report_format == FLVMETA_FORMAT_JSON) {
@@ -157,12 +158,15 @@ static void report_print_message(
         else {
             /* raw report entry */
             /*printf("%s:", opts->input_file);*/
-            printf("0x%.8" FILE_OFFSET_PRINTF_FORMAT "x: ", offset);
+            printf("0x%.8" FILE_OFFSET_PRINTF_FORMAT "x: ", FILE_OFFSET_PRINTF_TYPE(offset));
             printf("%s %s: %s\n", levelstr, code, message);
         }
     }
 }
 
+/* timestamp distance */
+#define timestamp_distance(t1, t2) ((t1)>(t2)?(t1)-(t2):(t2)-(t1))
+
 /* convenience macros */
 #define print_info(code, offset, message) \
     report_print_message(FLVMETA_CHECK_LEVEL_INFO, code, offset, message, opts, &ctxt)
@@ -201,12 +205,12 @@ int check_flv_file(const flvmeta_opts * opts) {
     flv_stream * flv_in;
     flv_header header;
     check_context ctxt;
-    int errors, warnings;
+    uint32 errors, warnings;
     int result;
     char message[256];
     uint32 prev_tag_size, tag_number;
     uint32 last_timestamp, last_video_timestamp, last_audio_timestamp;
-    struct stat file_stats;
+    file_offset_t filesize;
     int have_audio, have_video;
     flvmeta_opts opts_loc;
     flv_info info;
@@ -221,12 +225,14 @@ int check_flv_file(const flvmeta_opts * opts) {
     flv_audio_tag prev_audio_tag;
     int have_prev_video_tag;
     flv_video_tag prev_video_tag;
+    
+    int consecutive_unknown_tags;
 
     int video_frames_number, keyframes_number;
 
     prev_audio_tag = 0;
     prev_video_tag = 0;
-    
+
     have_audio = have_video = 0;
     tag_number = 0;
     last_timestamp = last_video_timestamp = last_audio_timestamp = 0;
@@ -238,9 +244,10 @@ int check_flv_file(const flvmeta_opts * opts) {
     on_metadata = on_metadata_name = NULL;
     have_on_last_second = 0;
     on_last_second_timestamp = 0;
+    consecutive_unknown_tags = 0;
 
     /* file stats */
-    if (stat(opts->input_file, &file_stats) != 0) {
+    if (flvmeta_filesize(opts->input_file, &filesize) == 0) {
         return ERROR_OPEN_READ;
     }
 
@@ -249,7 +256,7 @@ int check_flv_file(const flvmeta_opts * opts) {
     if (flv_in == NULL) {
         return ERROR_OPEN_READ;
     }
-    
+
     errors = warnings = 0;
 
     report_start(opts, &ctxt);
@@ -269,7 +276,7 @@ int check_flv_file(const flvmeta_opts * opts) {
 
     /* version */
     if (header.version != FLV_VERSION) {
-        sprintf(message, "header version should be 1, %d found instead", header.version);
+        sprintf(message, "header version should be 1, %u found instead", header.version);
         print_error(ERROR_HEADER_BAD_VERSION, 3, message);
     }
 
@@ -291,7 +298,7 @@ int check_flv_file(const flvmeta_opts * opts) {
 
     /* offset */
     if (flv_header_get_offset(header) != 9) {
-        sprintf(message, "header offset should be 9, %d found instead", flv_header_get_offset(header));
+        sprintf(message, "header offset should be 9, %u found instead", flv_header_get_offset(header));
         print_error(ERROR_HEADER_BAD_OFFSET, 5, message);
     }
 
@@ -303,18 +310,18 @@ int check_flv_file(const flvmeta_opts * opts) {
         goto end;
     }
     else if (prev_tag_size != 0) {
-        sprintf(message, "first previous tag size should be 0, %d found instead", prev_tag_size);
+        sprintf(message, "first previous tag size should be 0, %u found instead", prev_tag_size);
         print_error(ERROR_PREV_TAG_SIZE_BAD_FIRST, 9, message);
     }
 
     /* we reached the end of file: no tags in file */
-    if (flv_get_offset(flv_in) == file_stats.st_size) {
+    if (flv_get_offset(flv_in) == filesize) {
         print_fatal(FATAL_GENERAL_NO_TAG, 13, "file does not contain tags");
         goto end;
     }
 
     /** read tags **/
-    while (flv_get_offset(flv_in) < file_stats.st_size) {
+    while (flv_get_offset(flv_in) < filesize) {
         flv_tag tag;
         file_offset_t offset;
         uint32 body_length, timestamp, stream_id;
@@ -338,8 +345,17 @@ int check_flv_file(const flvmeta_opts * opts) {
             && tag.type != FLV_TAG_TYPE_VIDEO
             && tag.type != FLV_TAG_TYPE_META
         ) {
-            sprintf(message, "unknown tag type %hhd", tag.type);
+            sprintf(message, "unknown tag type %" PRI_BYTE "d", tag.type);
             print_error(ERROR_TAG_TYPE_UNKNOWN, offset, message);
+            ++consecutive_unknown_tags;
+
+            if (consecutive_unknown_tags >= 2) {
+                print_fatal(FATAL_CONSECUTIVE_UNKNOWN_TAGS, offset, "consecutive tags with unknown type found, aborting");
+                goto end;
+            }
+        }
+        else {
+            consecutive_unknown_tags = 0;
         }
 
         /* check consistency with global header */
@@ -357,13 +373,13 @@ int check_flv_file(const flvmeta_opts * opts) {
         }
 
         /* check body length */
-        if (body_length > (file_stats.st_size - flv_get_offset(flv_in))) {
-            sprintf(message, "tag body length (%d bytes) exceeds file size", body_length);
+        if (body_length > (filesize - flv_get_offset(flv_in))) {
+            sprintf(message, "tag body length (%u bytes) exceeds file size", body_length);
             print_fatal(FATAL_TAG_BODY_LENGTH_OVERFLOW, offset + 1, message);
             goto end;
         }
         else if (body_length > MAX_ACCEPTABLE_TAG_BODY_LENGTH) {
-            sprintf(message, "tag body length (%d bytes) is abnormally large", body_length);
+            sprintf(message, "tag body length (%u bytes) is abnormally large", body_length);
             print_warning(WARNING_TAG_BODY_LENGTH_LARGE, offset + 1, message);
         }
         else if (body_length == 0) {
@@ -375,14 +391,14 @@ int check_flv_file(const flvmeta_opts * opts) {
 
         /* check whether first timestamp is zero */
         if (tag_number == 1 && timestamp != 0) {
-            sprintf(message, "first timestamp should be zero, %d found instead", timestamp);
+            sprintf(message, "first timestamp should be zero, %u found instead", timestamp);
             print_error(ERROR_TIMESTAMP_FIRST_NON_ZERO, offset + 4, message);
         }
 
         /* check whether timestamps decrease in a given stream */
         if (tag.type == FLV_TAG_TYPE_AUDIO) {
             if (last_audio_timestamp > timestamp) {
-                sprintf(message, "audio tag timestamps are decreasing from %d to %d", last_audio_timestamp, timestamp);
+                sprintf(message, "audio tag timestamps are decreasing from %u to %u", last_audio_timestamp, timestamp);
                 print_error(ERROR_TIMESTAMP_AUDIO_DECREASE, offset + 4, message);
             }
             last_audio_timestamp = timestamp;
@@ -390,7 +406,7 @@ int check_flv_file(const flvmeta_opts * opts) {
         }
         if (tag.type == FLV_TAG_TYPE_VIDEO) {
             if (last_video_timestamp > timestamp) {
-                sprintf(message, "video tag timestamps are decreasing from %d to %d", last_video_timestamp, timestamp);
+                sprintf(message, "video tag timestamps are decreasing from %u to %u", last_video_timestamp, timestamp);
                 print_error(ERROR_TIMESTAMP_VIDEO_DECREASE, offset + 4, message);
             }
             last_video_timestamp = timestamp;
@@ -404,23 +420,23 @@ int check_flv_file(const flvmeta_opts * opts) {
 
         /* check whether timestamps decrease globally */
         else if (!decr_timestamp_signaled && last_timestamp > timestamp && last_timestamp - timestamp >= 1000) {
-            sprintf(message, "timestamps are decreasing from %d to %d", last_timestamp, timestamp);
+            sprintf(message, "timestamps are decreasing from %u to %u", last_timestamp, timestamp);
             print_error(ERROR_TIMESTAMP_DECREASE, offset + 4, message);
         }
 
         last_timestamp = timestamp;
 
         /* check for desyncs between audio and video: one second or more is suspicious */
-        if (have_video && have_audio && !have_desync && abs(last_video_timestamp - last_audio_timestamp) >= 1000) {
+        if (have_video && have_audio && !have_desync && timestamp_distance(last_video_timestamp, last_audio_timestamp) >= 1000) {
             sprintf(message, "audio and video streams are desynchronized by %d ms",
-                abs(last_video_timestamp - last_audio_timestamp));
+                timestamp_distance(last_video_timestamp, last_audio_timestamp));
             print_warning(WARNING_TIMESTAMP_DESYNC, offset + 4, message);
             have_desync = 1; /* do not repeat */
         }
 
         /** stream id must be zero **/
         if (stream_id != 0) {
-            sprintf(message, "tag stream id must be zero, %d found instead", stream_id);
+            sprintf(message, "tag stream id must be zero, %u found instead", stream_id);
             print_error(ERROR_TAG_STREAM_ID_NON_ZERO, offset + 8, message);
         }
 
@@ -446,7 +462,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                 /* check format */
                 audio_format = flv_audio_tag_sound_format(at);
                 if (audio_format == 12 || audio_format == 13) {
-                    sprintf(message, "unknown audio format %d", audio_format);
+                    sprintf(message, "unknown audio format %u", audio_format);
                     print_warning(WARNING_AUDIO_CODEC_UNKNOWN, offset + 11, message);
                 }
                 else if (audio_format == FLV_AUDIO_TAG_SOUND_FORMAT_G711_A
@@ -455,7 +471,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                     || audio_format == FLV_AUDIO_TAG_SOUND_FORMAT_MP3_8
                     || audio_format == FLV_AUDIO_TAG_SOUND_FORMAT_DEVICE_SPECIFIC
                 ) {
-                    sprintf(message, "audio format %d is reserved for internal use", audio_format);
+                    sprintf(message, "audio format %u is reserved for internal use", audio_format);
                     print_warning(WARNING_AUDIO_CODEC_RESERVED, offset + 11, message);
                 }
 
@@ -513,7 +529,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                     && video_frame_type != FLV_VIDEO_TAG_FRAME_TYPE_GENERATED_KEYFRAME
                     && video_frame_type != FLV_VIDEO_TAG_FRAME_TYPE_COMMAND_FRAME
                 ) {
-                    sprintf(message, "unknown video frame type %d", video_frame_type);
+                    sprintf(message, "unknown video frame type %u", video_frame_type);
                     print_error(ERROR_VIDEO_FRAME_TYPE_UNKNOWN, offset + 11, message);
                 }
 
@@ -536,7 +552,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                     && video_codec != FLV_VIDEO_TAG_CODEC_SCREEN_VIDEO_V2
                     && video_codec != FLV_VIDEO_TAG_CODEC_AVC
                 ) {
-                    sprintf(message, "unknown video codec id %d", video_codec);
+                    sprintf(message, "unknown video codec id %u", video_codec);
                     print_error(ERROR_VIDEO_CODEC_UNKNOWN, offset + 11, message);
                 }
 
@@ -574,7 +590,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                 }
                 else if (amf_data_get_type(name) != AMF_TYPE_STRING) {
                     /* name type checking */
-                    sprintf(message, "invalid metadata name type: %d, should be a string (2)", amf_data_get_type(name));
+                    sprintf(message, "invalid metadata name type: %u, should be a string (2)", amf_data_get_type(name));
                     print_error(ERROR_METADATA_NAME_INVALID_TYPE, offset, message);
                 }
                 else {
@@ -585,11 +601,11 @@ int check_flv_file(const flvmeta_opts * opts) {
 
                     /* check whether all body size has been read */
                     if (flv_in->current_tag_body_length > 0) {
-                        sprintf(message, "%d bytes not read in tag body after metadata end", body_length - flv_in->current_tag_body_length);
+                        sprintf(message, "%u bytes not read in tag body after metadata end", body_length - flv_in->current_tag_body_length);
                         print_warning(WARNING_METADATA_DATA_REMAINING, flv_get_offset(flv_in), message);
                     }
                     else if (flv_in->current_tag_body_overflow > 0) {
-                        sprintf(message, "%d bytes missing from tag body after metadata end", flv_in->current_tag_body_overflow);
+                        sprintf(message, "%u bytes missing from tag body after metadata end", flv_in->current_tag_body_overflow);
                         print_warning(WARNING_METADATA_DATA_MISSING, flv_get_offset(flv_in), message);
                     }
 
@@ -614,7 +630,7 @@ int check_flv_file(const flvmeta_opts * opts) {
 
                             /* check onMetadata type */
                             if (amf_data_get_type(on_metadata) != AMF_TYPE_ASSOCIATIVE_ARRAY) {
-                                sprintf(message, "invalid onMetaData data type: %d, should be an associative array (8)", amf_data_get_type(on_metadata));
+                                sprintf(message, "invalid onMetaData data type: %u, should be an associative array (8)", amf_data_get_type(on_metadata));
                                 print_error(ERROR_METADATA_DATA_INVALID_TYPE, offset, message);
                             }
 
@@ -653,7 +669,7 @@ int check_flv_file(const flvmeta_opts * opts) {
         }
 
         if (prev_tag_size != FLV_TAG_SIZE + body_length) {
-            sprintf(message, "previous tag size should be %d, %d found instead", FLV_TAG_SIZE + body_length, prev_tag_size);
+            sprintf(message, "previous tag size should be %u, %u found instead", FLV_TAG_SIZE + body_length, prev_tag_size);
             print_error(ERROR_PREV_TAG_SIZE_BAD, flv_get_offset(flv_in), message);
         }
     }
@@ -669,39 +685,39 @@ int check_flv_file(const flvmeta_opts * opts) {
     }
 
     /* check last timestamps */
-    if (have_video && have_audio && abs(last_audio_timestamp - last_video_timestamp) >= 1000) {
+    if (have_video && have_audio && timestamp_distance(last_audio_timestamp, last_video_timestamp) >= 1000) {
         if (last_audio_timestamp > last_video_timestamp) {
-            sprintf(message, "video stops %d ms before audio", last_audio_timestamp - last_video_timestamp);
-            print_warning(WARNING_TIMESTAMP_VIDEO_ENDS_FIRST, file_stats.st_size, message);
+            sprintf(message, "video stops %u ms before audio", last_audio_timestamp - last_video_timestamp);
+            print_warning(WARNING_TIMESTAMP_VIDEO_ENDS_FIRST, filesize, message);
         }
         else {
-            sprintf(message, "audio stops %d ms before video", last_video_timestamp - last_audio_timestamp);
-            print_warning(WARNING_TIMESTAMP_AUDIO_ENDS_FIRST, file_stats.st_size, message);
+            sprintf(message, "audio stops %u ms before video", last_video_timestamp - last_audio_timestamp);
+            print_warning(WARNING_TIMESTAMP_AUDIO_ENDS_FIRST, filesize, message);
         }
     }
 
     /* check video keyframes */
     if (have_video && keyframes_number == 0) {
-        print_warning(WARNING_VIDEO_NO_KEYFRAME, file_stats.st_size, "no keyframe detected, file is probably broken or incomplete");
+        print_warning(WARNING_VIDEO_NO_KEYFRAME, filesize, "no keyframe detected, file is probably broken or incomplete");
     }
     if (have_video && keyframes_number == video_frames_number) {
-        print_warning(WARNING_VIDEO_ONLY_KEYFRAMES, file_stats.st_size, "only keyframes detected, probably inefficient compression scheme used");
+        print_warning(WARNING_VIDEO_ONLY_KEYFRAMES, filesize, "only keyframes detected, probably inefficient compression scheme used");
     }
 
     /* only keyframes + onLastSecond bug */
     if (have_video && have_on_last_second && keyframes_number == video_frames_number) {
-        print_warning(WARNING_VIDEO_ONLY_KF_LAST_SEC, file_stats.st_size, "only keyframes detected and onLastSecond event present, file is probably not playable");
+        print_warning(WARNING_VIDEO_ONLY_KF_LAST_SEC, filesize, "only keyframes detected and onLastSecond event present, file is probably not playable");
     }
 
     /* check onLastSecond timestamp */
     if (have_on_last_second && (last_timestamp - on_last_second_timestamp) >= 2000) {
-        sprintf(message, "onLastSecond event located %d ms before the last tag", last_timestamp - on_last_second_timestamp);
-        print_warning(WARNING_METADATA_LAST_SECOND_BAD, file_stats.st_size, message);
+        sprintf(message, "onLastSecond event located %u ms before the last tag", last_timestamp - on_last_second_timestamp);
+        print_warning(WARNING_METADATA_LAST_SECOND_BAD, filesize, message);
     }
 
     /* check onMetaData presence */
     if (!have_on_metadata) {
-        print_warning(WARNING_METADATA_NOT_PRESENT, file_stats.st_size, "onMetaData event not found, file might not be playable");
+        print_warning(WARNING_METADATA_NOT_PRESENT, filesize, "onMetaData event not found, file might not be playable");
     }
     else {
         amf_node * n;
@@ -863,7 +879,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                         width = info.video_width;
                         file_width = amf_number_get_value(data);
 
-                        if (fabs(file_width - width) >= 1.0) {
+                        if (fabs(file_width - width) >= 1.0 && width != 0) {
                             sprintf(message, "width should be %.12g, got %.12g", width, file_width);
                             print_warning(WARNING_AMF_DATA_INVALID_VALUE, on_metadata_offset, message);
                         }
@@ -889,7 +905,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                         height = info.video_height;
                         file_height = amf_number_get_value(data);
 
-                        if (fabs(file_height - height) >= 1.0) {
+                        if (fabs(file_height - height) >= 1.0 && height != 0) {
                             sprintf(message, "height should be %.12g, got %.12g", height, file_height);
                             print_warning(WARNING_AMF_DATA_INVALID_VALUE, on_metadata_offset, message);
                         }
@@ -1071,13 +1087,13 @@ int check_flv_file(const flvmeta_opts * opts) {
             /* filesize: (number) */
             if (!strcmp((char*)name, "filesize")) {
                 if (type == AMF_TYPE_NUMBER) {
-                    number64 filesize, file_filesize;
+                    number64 real_filesize, file_filesize;
 
-                    filesize = (number64)(file_stats.st_size);
+                    real_filesize = (number64)(filesize);
                     file_filesize = amf_number_get_value(data);
 
-                    if (fabs(file_filesize - filesize) >= 1.0) {
-                        sprintf(message, "filesize should be %.12g, got %.12g", filesize, file_filesize);
+                    if (fabs(file_filesize - real_filesize) >= 1.0) {
+                        sprintf(message, "filesize should be %.12g, got %.12g", real_filesize, file_filesize);
                         print_warning(WARNING_AMF_DATA_INVALID_VALUE, on_metadata_offset, message);
                     }
                 }
@@ -1144,7 +1160,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                 if (type == AMF_TYPE_NUMBER) {
                     number64 datasize, file_datasize;
                     uint32 on_metadata_size;
-                    
+
                     on_metadata_size = FLV_TAG_SIZE +
                         (uint32)(amf_data_size(on_metadata_name) + amf_data_size(on_metadata));
                     datasize = (number64)(info.meta_data_size + on_metadata_size);
@@ -1274,7 +1290,7 @@ int check_flv_file(const flvmeta_opts * opts) {
             if (!strcmp((char*)name, "keyframes")) {
                 if (type == AMF_TYPE_OBJECT) {
                     amf_data * file_times, * file_filepositions;
-                    
+
                     file_times = amf_object_get(data, "times");
                     file_filepositions = amf_object_get(data, "filepositions");
 
@@ -1347,7 +1363,7 @@ int check_flv_file(const flvmeta_opts * opts) {
                                                 time, f_time);
                                             print_warning(WARNING_KEYFRAMES_TIME_BAD, on_metadata_offset, message);
                                         }
-                                        
+
                                         /* check for duplicate time, can happen in H.264 files */
                                         if (have_last_time && last_file_time == f_time) {
                                             sprintf(message, "Duplicate keyframe time: %.12g", f_time);
@@ -1411,7 +1427,7 @@ int check_flv_file(const flvmeta_opts * opts) {
 
     /* could we compute video resolution ? */
     if (info.video_width == 0 && info.video_height == 0) {
-        print_warning(WARNING_VIDEO_SIZE_ERROR, file_stats.st_size, "unable to determine video resolution");
+        print_warning(WARNING_VIDEO_SIZE_ERROR, filesize, "unable to determine video resolution");
     }
 
     /* global info */
@@ -1440,16 +1456,16 @@ int check_flv_file(const flvmeta_opts * opts) {
     }
 
     /* is the file larger than 4GB ? */
-    if (file_stats.st_size > 0xFFFFFFFFULL) {
+    if (filesize > 0xFFFFFFFFULL) {
         print_info(INFO_GENERAL_LARGE_FILE, 0, "file is larger than 4 GB");
     }
 
 end:
     report_end(opts, &ctxt, errors, warnings);
-    
+
     amf_data_free(on_metadata);
     amf_data_free(on_metadata_name);
     flv_close(flv_in);
-    
+
     return (errors > 0) ? ERROR_INVALID_FLV_FILE : OK;
 }
