@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -51,8 +51,20 @@ const char * dump_string_get_video_codec(flv_video_tag tag) {
 
 const char * dump_string_get_video_frame_type(flv_video_tag tag) {
     switch (flv_video_tag_frame_type(tag)) {
-        case FLV_VIDEO_TAG_FRAME_TYPE_KEYFRAME: return "keyframe";
-        case FLV_VIDEO_TAG_FRAME_TYPE_INTERFRAME: return "inter frame";
+        case FLV_VIDEO_TAG_FRAME_TYPE_KEYFRAME:
+            if (flv_video_tag_codec_id(tag) == FLV_VIDEO_TAG_CODEC_AVC) {
+                return "seekable frame";
+            }
+            else {
+                return "keyframe";
+            }
+        case FLV_VIDEO_TAG_FRAME_TYPE_INTERFRAME:
+            if (flv_video_tag_codec_id(tag) == FLV_VIDEO_TAG_CODEC_AVC) {
+                return "non-seekable frame";
+            }
+            else {
+                return "inter frame";
+            }
         case FLV_VIDEO_TAG_FRAME_TYPE_DISPOSABLE_INTERFRAME: return "disposable inter frame";
         case FLV_VIDEO_TAG_FRAME_TYPE_GENERATED_KEYFRAME: return "generated keyframe";
         case FLV_VIDEO_TAG_FRAME_TYPE_COMMAND_FRAME: return "video info/command frame";
@@ -60,6 +72,15 @@ const char * dump_string_get_video_frame_type(flv_video_tag tag) {
     }
 }
 
+const char * dump_string_get_avc_packet_type(flv_avc_packet_type type) {
+    switch (type) {
+        case FLV_AVC_PACKET_TYPE_SEQUENCE_HEADER: return "AVC sequence header";
+        case FLV_AVC_PACKET_TYPE_NALU: return "AVC NALU";
+        case FLV_AVC_PACKET_TYPE_SEQUENCE_END: return "AVC sequence end";
+        default: return "Unknown";
+    }
+}
+
 const char * dump_string_get_sound_type(flv_audio_tag tag) {
     switch (flv_audio_tag_sound_type(tag)) {
         case FLV_AUDIO_TAG_SOUND_TYPE_MONO: return "mono";
@@ -106,6 +127,14 @@ const char * dump_string_get_sound_format(flv_audio_tag tag) {
     }
 }
 
+const char * dump_string_get_aac_packet_type(flv_aac_packet_type type) {
+    switch (type) {
+        case FLV_AAC_PACKET_TYPE_SEQUENCE_HEADER: return "AAC sequence header";
+        case FLV_AAC_PACKET_TYPE_RAW: return "AAC raw";
+        default: return "Unknown";
+    }
+}
+
 /* dump metadata from a FLV file */
 int dump_metadata(const flvmeta_opts * options) {
     int retval;
