@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -145,6 +145,37 @@ static int json_on_video_tag(flv_tag * tag, flv_video_tag vt, flv_parser * parse
     json_emit_string_z(je, dump_string_get_video_codec(vt));
     json_emit_object_key_z(je, "frameType");
     json_emit_string_z(je, dump_string_get_video_frame_type(vt));
+
+    /* if AVC, detect frame type and composition time */
+    if (flv_video_tag_codec_id(vt) == FLV_VIDEO_TAG_CODEC_AVC) {
+        flv_avc_packet_type type;
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_avc_packet_type)) < sizeof(flv_avc_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        json_emit_object_key_z(je, "AVCData");
+
+        json_emit_object_start(je);
+        json_emit_object_key_z(je, "packetType");
+        json_emit_string_z(je, dump_string_get_avc_packet_type(type));
+
+        /* composition time */
+        if (type == FLV_AVC_PACKET_TYPE_NALU) {
+            uint24_be composition_time;
+
+            if (flv_read_tag_body(parser->stream, &composition_time, sizeof(uint24_be)) < sizeof(uint24_be)) {
+                return ERROR_INVALID_TAG;
+            }
+
+            json_emit_object_key_z(je, "compositionTimeOffset");
+            json_emit_integer(je, uint24_be_to_uint32(composition_time));
+        }
+
+        json_emit_object_end(je);
+    }
+
     json_emit_object_end(je);
 
     return OK;
@@ -164,6 +195,26 @@ static int json_on_audio_tag(flv_tag * tag, flv_audio_tag at, flv_parser * parse
     json_emit_string_z(je, dump_string_get_sound_rate(at));
     json_emit_object_key_z(je, "format");
     json_emit_string_z(je, dump_string_get_sound_format(at));
+
+    /* if AAC, detect packet type */
+    if (flv_audio_tag_sound_format(at) == FLV_AUDIO_TAG_SOUND_FORMAT_AAC) {
+        flv_aac_packet_type type;
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_aac_packet_type)) < sizeof(flv_aac_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        json_emit_object_key_z(je, "AACData");
+
+        json_emit_object_start(je);
+        json_emit_object_key_z(je, "packetType");
+
+        json_emit_string_z(je, dump_string_get_aac_packet_type(type));
+
+        json_emit_object_end(je);
+    }
+
     json_emit_object_end(je);
 
     return OK;
@@ -187,9 +238,9 @@ static int json_on_metadata_tag(flv_tag * tag, amf_data * name, amf_data * data,
 static int json_on_prev_tag_size(uint32 size, flv_parser * parser) {
     json_emitter * je;
     je = (json_emitter*)parser->user_data;
-    
+
     json_emit_object_end(je);
-    
+
     return OK;
 }
 
@@ -231,7 +282,7 @@ void dump_json_setup_metadata_dump(flv_parser * parser) {
 
 int dump_json_file(flv_parser * parser, const flvmeta_opts * options) {
     json_emitter je;
-    
+
     parser->on_header = json_on_header;
     parser->on_tag = json_on_tag;
     parser->on_audio_tag = json_on_audio_tag;
