@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -29,7 +29,7 @@
 
 static int raw_on_header(flv_header * header, flv_parser * parser) {
     int * n;
-    
+
     n = (int*) malloc(sizeof(uint32));
     if (n == NULL) {
         return ERROR_MEMORY;
@@ -39,7 +39,7 @@ static int raw_on_header(flv_header * header, flv_parser * parser) {
     *n = 0;
 
     printf("Magic: %.3s\n", header->signature);
-    printf("Version: %d\n", header->version);
+    printf("Version: %" PRI_BYTE "u\n", header->version);
     printf("Has audio: %s\n", flv_header_has_audio(*header) ? "yes" : "no");
     printf("Has video: %s\n", flv_header_has_video(*header) ? "yes" : "no");
     printf("Offset: %u\n", swap_uint32(header->offset));
@@ -53,8 +53,8 @@ static int raw_on_tag(flv_tag * tag, flv_parser * parser) {
     n = ((int*)parser->user_data);
     ++(*n);
 
-    printf("--- Tag #%u at 0x%" FILE_OFFSET_PRINTF_FORMAT "X", *n, parser->stream->current_tag_offset);
-    printf(" (%" FILE_OFFSET_PRINTF_FORMAT "i) ---\n", parser->stream->current_tag_offset);
+    printf("--- Tag #%u at 0x%" FILE_OFFSET_PRINTF_FORMAT "X", *n, FILE_OFFSET_PRINTF_TYPE(parser->stream->current_tag_offset));
+    printf(" (%" FILE_OFFSET_PRINTF_FORMAT "u) ---\n", FILE_OFFSET_PRINTF_TYPE(parser->stream->current_tag_offset));
     printf("Tag type: %s\n", dump_string_get_tag_type(tag));
     printf("Body length: %u\n", flv_tag_get_body_length(*tag));
     printf("Timestamp: %u\n", flv_tag_get_timestamp(*tag));
@@ -65,6 +65,30 @@ static int raw_on_tag(flv_tag * tag, flv_parser * parser) {
 static int raw_on_video_tag(flv_tag * tag, flv_video_tag vt, flv_parser * parser) {
     printf("* Video codec: %s\n", dump_string_get_video_codec(vt));
     printf("* Video frame type: %s\n", dump_string_get_video_frame_type(vt));
+
+    /* if AVC, detect frame type and composition time */
+    if (flv_video_tag_codec_id(vt) == FLV_VIDEO_TAG_CODEC_AVC) {
+        flv_avc_packet_type type;
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_avc_packet_type)) < sizeof(flv_avc_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        printf("* AVC packet type: %s\n", dump_string_get_avc_packet_type(type));
+
+        /* composition time */
+        if (type == FLV_AVC_PACKET_TYPE_NALU) {
+            uint24_be composition_time;
+
+            if (flv_read_tag_body(parser->stream, &composition_time, sizeof(uint24_be)) < sizeof(uint24_be)) {
+                return ERROR_INVALID_TAG;
+            }
+
+            printf("* Composition time offset: %i\n", uint24_be_to_uint32(composition_time));
+        }
+    }
+
     return OK;
 }
 
@@ -73,6 +97,19 @@ static int raw_on_audio_tag(flv_tag * tag, flv_audio_tag at, flv_parser * parser
     printf("* Sound size: %s\n", dump_string_get_sound_size(at));
     printf("* Sound rate: %s\n", dump_string_get_sound_rate(at));
     printf("* Sound format: %s\n", dump_string_get_sound_format(at));
+
+    /* if AAC, detect packet type */
+    if (flv_audio_tag_sound_format(at) == FLV_AUDIO_TAG_SOUND_FORMAT_AAC) {
+        flv_aac_packet_type type;
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_aac_packet_type)) < sizeof(flv_aac_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        printf("* AAC packet type: %s\n", dump_string_get_aac_packet_type(type));
+    }
+
     return OK;
 }
 
@@ -128,7 +165,7 @@ int dump_raw_file(flv_parser * parser, const flvmeta_opts * options) {
     parser->on_metadata_tag = raw_on_metadata_tag;
     parser->on_prev_tag_size = raw_on_prev_tag_size;
     parser->on_stream_end = raw_on_stream_end;
-    
+
     return flv_parse(options->input_file, parser);
 }
 
