@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -162,7 +162,7 @@ static void xml_amf_data_dump(const amf_data * data, int qualified, int indent_l
 
 static int xml_on_header(flv_header * header, flv_parser * parser) {
     puts("<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>");
-    printf("<flv xmlns=\"http://schemas.flvmeta.org/FLV/1.0/\" xmlns:amf=\"http://schemas.flvmeta.org/AMF0/1.0/\" hasVideo=\"%s\" hasAudio=\"%s\" version=\"%i\">\n",
+    printf("<flv xmlns=\"http://schemas.flvmeta.org/FLV/1.0/\" xmlns:amf=\"http://schemas.flvmeta.org/AMF0/1.0/\" hasVideo=\"%s\" hasAudio=\"%s\" version=\"%" PRI_BYTE "u\">\n",
         flv_header_has_video(*header) ? "true" : "false",
         flv_header_has_audio(*header) ? "true" : "false",
         header->version);
@@ -174,15 +174,47 @@ static int xml_on_tag(flv_tag * tag, flv_parser * parser) {
         dump_string_get_tag_type(tag),
         flv_tag_get_timestamp(*tag),
         flv_tag_get_body_length(*tag));
-    printf(" offset=\"%" FILE_OFFSET_PRINTF_FORMAT "i\">\n",
-        parser->stream->current_tag_offset);
+    printf(" offset=\"%" FILE_OFFSET_PRINTF_FORMAT "u\">\n",
+        FILE_OFFSET_PRINTF_TYPE(parser->stream->current_tag_offset));
 
     return OK;
 }
 
 static int xml_on_video_tag(flv_tag * tag, flv_video_tag vt, flv_parser * parser) {
     printf("    <videoData codecID=\"%s\"", dump_string_get_video_codec(vt));
-    printf(" frameType=\"%s\"/>\n", dump_string_get_video_frame_type(vt));
+    printf(" frameType=\"%s\"", dump_string_get_video_frame_type(vt));
+
+    /* if AVC, detect frame type and composition time */
+    if (flv_video_tag_codec_id(vt) == FLV_VIDEO_TAG_CODEC_AVC) {
+        flv_avc_packet_type type;
+
+        printf(">\n");
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_avc_packet_type)) < sizeof(flv_avc_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        printf("        <AVCData packetType=\"%s\"", dump_string_get_avc_packet_type(type));
+
+        /* composition time */
+        if (type == FLV_AVC_PACKET_TYPE_NALU) {
+            uint24_be composition_time;
+
+            if (flv_read_tag_body(parser->stream, &composition_time, sizeof(uint24_be)) < sizeof(uint24_be)) {
+                return ERROR_INVALID_TAG;
+            }
+
+            printf(" compositionTimeOffset=\"%i\"", uint24_be_to_uint32(composition_time));
+        }
+
+        printf("/>\n");
+        printf("    </videoData>\n");
+    }
+    else {
+        printf("/>\n");
+    }
+
     return OK;
 }
 
@@ -190,7 +222,26 @@ static int xml_on_audio_tag(flv_tag * tag, flv_audio_tag at, flv_parser * parser
     printf("    <audioData type=\"%s\"", dump_string_get_sound_type(at));
     printf(" size=\"%s\"", dump_string_get_sound_size(at));
     printf(" rate=\"%s\"", dump_string_get_sound_rate(at));
-    printf(" format=\"%s\"/>\n", dump_string_get_sound_format(at));
+    printf(" format=\"%s\"", dump_string_get_sound_format(at));
+
+    /* if AAC, detect packet type */
+    if (flv_audio_tag_sound_format(at) == FLV_AUDIO_TAG_SOUND_FORMAT_AAC) {
+        flv_aac_packet_type type;
+
+        printf(">\n");
+
+        /* packet type */
+        if (flv_read_tag_body(parser->stream, &type, sizeof(flv_aac_packet_type)) < sizeof(flv_aac_packet_type)) {
+            return ERROR_INVALID_TAG;
+        }
+
+        printf("        <AACData packetType=\"%s\"/>\n", dump_string_get_aac_packet_type(type));
+        printf("    </audioData>\n");
+    }
+    else {
+        printf("/>\n");
+    }
+
     return OK;
 }
 
