@@ -1,7 +1,7 @@
 /*
     FLVMeta - FLV Metadata Editor
 
-    Copyright (C) 2007-2013 Marc Noirot <marc.noirot AT gmail.com>
+    Copyright (C) 2007-2016 Marc Noirot <marc.noirot AT gmail.com>
 
     This file is part of FLVMeta.
 
@@ -22,12 +22,41 @@
 #ifndef __UTIL_H__
 #define __UTIL_H__
 
+#include <stdio.h>
+
+#include "types.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 /* determine whether two paths physically point to the same file */
-int same_file(const char * file1, const char * file2);
+int flvmeta_same_file(const char * file1, const char * file2);
+
+#ifdef WIN32
+/*
+    Returns a descriptor to a temporary file.
+    This function is meant as a Windows replacement
+    to the broken standard tmpfile() function.
+*/
+FILE * flvmeta_tmpfile(void);
+
+#else /* WIN32 */
+# define flvmeta_tmpfile tmpfile
+#endif /* WIN32 */
+
+/*
+    File size (LFS compatible).
+    Returns a non-zero value if successful, zero otherwise.
+*/
+int flvmeta_filesize(const char * filename, file_offset_t * filesize);
+
+#ifndef HAVE_ISFINITE
+/*
+    Check whether a double is finite (not infinity or NaN)
+*/
+int flvmeta_isfinite(double d);
+#endif /* HAVE_ISFINITE */
 
 #ifdef __cplusplus
 }
